'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = parseEffects;

var _addEffect = require('./add-effect');

var _addEffect2 = _interopRequireDefault(_addEffect);

var _mergeColors = require('../utils/merge-colors');

var _mergeColors2 = _interopRequireDefault(_mergeColors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parseEffects(elem, config) {
    var type = elem.type;
    var _effects = elem._effects;

    if (type !== 'text' && type !== 'image') {
        return elem;
    }

    if (type === 'text') {
        elem.textEffects = [];
    }

    if (type === 'image') {
        elem.imageEffects = [];
    }

    var typeEffects = elem.textEffects || elem.imageEffects;

    Object.keys(_effects).forEach(function (key) {
        if (key === 'shadow') {
            return;
        }
        var effects = _effects[key];
        effects.forEach(function (effect) {
            if (key === 'Gradient Overlay') {
                if (!effect.enable) {
                    return;
                }
                key = 'filling';

                var stops = effect.stops;
                stops.forEach(function (stop) {
                    stop.color = (0, _mergeColors2.default)(elem.color || '#ffffff', stop.color);
                });

                effect = {
                    type: 2,
                    gradient: effect
                };
            }

            if (key === 'Color Overlay') {
                key = 'filling';
                effect.type = 0;
            }

            if (key === 'Outer Glow') {
                key = 'shadow';
            }

            if (type === 'image' && key === 'stroke') {
                var width = effect.width;
                effect.width = effect.type === 'outer' ? width : width / 2;
            }

            effect.enable = true;

            var result = (0, _addEffect2.default)({
                key: key,
                data: effect,
                effects: typeEffects
            });

            typeEffects = result;
        });
    });

    var shadowEffects = _effects['shadow'];
    if (shadowEffects) {
        shadowEffects.forEach(function (effect) {
            effect.enable = true;
            var result = (0, _addEffect2.default)({
                key: 'shadow',
                data: effect,
                pos: -1,
                effects: typeEffects
            });

            typeEffects = result;
        });
    }

    delete elem.shadow;
    delete elem.stroke;

    return elem;
};
module.exports = exports['default'];