'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _psd = require('psd.js');

var _psd2 = _interopRequireDefault(_psd);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _lodash = require('lodash.camelcase');

var _lodash2 = _interopRequireDefault(_lodash);

var _createSvgPath = require('./parse-tools/create-svg-path');

var _createSvgPath2 = _interopRequireDefault(_createSvgPath);

var _standardLayerProps = require('./parse-tools/standard-layer-props');

var _standardLayerProps2 = _interopRequireDefault(_standardLayerProps);

var _parseTransform = require('./parse-tools/parse-transform');

var _parseTransform2 = _interopRequireDefault(_parseTransform);

var _parseTextContent = require('./parse-tools/parse-text-content');

var _parseTextContent2 = _interopRequireDefault(_parseTextContent);

var _getStyle = require('./parse-tools/get-style');

var _getStyle2 = _interopRequireDefault(_getStyle);

var _getFont = require('./parse-tools/get-font');

var _getFont2 = _interopRequireDefault(_getFont);

var _imageToBase = require('./parse-tools/image-to-base64');

var _imageToBase2 = _interopRequireDefault(_imageToBase);

var _mergeMasks = require('./parse-tools/merge-masks');

var _mergeMasks2 = _interopRequireDefault(_mergeMasks);

var _flatLayers = require('./parse-tools/flat-layers');

var _flatLayers2 = _interopRequireDefault(_flatLayers);

var _stringifyRgba = require('./utils/stringify-rgba');

var _stringifyRgba2 = _interopRequireDefault(_stringifyRgba);

var _decomposeTransform = require('./utils/decompose-transform');

var _decomposeTransform2 = _interopRequireDefault(_decomposeTransform);

var _rect = require('./utils/rect');

var _colorOverlay = require('./utils/color-overlay');

var _colorOverlay2 = _interopRequireDefault(_colorOverlay);

var _parseSvgLayer = require('./parse-tools/parse-svg-layer');
var _parseSvgLayer2 = _interopRequireDefault(_parseSvgLayer);

function _interopRequireDefault (obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray (arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var psdTools = {
  PSD: _psd2.default,
  options: {},
  collapseLayouts: null,
  layerProgress: {
    total: '-',
    currentPos: '-',
    layer: undefined
  },

  default: function _default (options) {
    var defaultOptions = {
      parseSVG: true,
      groupMode: 'flat',
      defaultTextType: 'block',
      imageQuality: 60,
      onProgress: function onProgress (data) { }
    };

    this.options = Object.assign({}, defaultOptions, options);
  },
  fromURL: function fromURL (url) {
    return _psd.fromURL(url)
  },
  parseFile: function parseFile (file, options) {
    this.default(options);

    var updateProgress = this.options.onProgress;

    if (!updateProgress) {
      updateProgress = function updateProgress () {
        return undefined;
      };
    };

    return _bluebird2.default.resolve().then(function () {
      updateProgress({
        message: '正在读取 PSD 树结构',
        progress: 0,
        layerProgress: Object.assign({}, psdTools.layerProgress)
      });

      return _psd2.default.fromDroppedFile(file);
    }).bind(this).then(function (psd) {
      updateProgress({
        message: '准备解析图层数据',
        progress: 10,
        layerProgress: Object.assign({}, psdTools.layerProgress)
      });

      return this.parsePSD(psd, function (data) {
        var len = data.length;
        var layerPos = data.count + '/' + len;
        var progress = 10 + 100 * 0.9 * (data.count / len);
        var name = data.layer.name;

        psdTools.layerProgress = Object.assign(psdTools.layerProgress, {
          total: len,
          currentPos: data.count,
          layer: data.layer
        });

        updateProgress({
          message: '正在解析图层(' + layerPos + '): ' + name,
          progress: progress,
          layerProgress: Object.assign({}, psdTools.layerProgress)
        });
      });
    }).delay(360).then(function (layouts) {
      updateProgress({
        message: 'PSD 解析完成',
        progress: 100,
        layerProgress: Object.assign({}, psdTools.layerProgress)
      });

      return layouts;
    });
  },
  parsePSD: function parsePSD (psd, dataProcesser) {
    if (!dataProcesser) {
      dataProcesser = function dataProcesser () {
        return undefined;
      };
    }

    return _bluebird2.default.try(function () {
      var tree = psd.tree();

      var groups = [];

      tree.children().map(function (layer, inx) {
        if (layer.type === 'group' && layer.visible()) {
          var layout_matches = layer.name.match(/\[\s*?(layout)\s*?\]\s*(.*?)?\s*$/i);
          if (layout_matches) {
            layer.name = layout_matches[2];
            if (layout_matches && layout_matches[1].toLowerCase() === 'layout') {
              groups.push(layer);
            }
          }
        }
      });

      if (!groups.length) {
        groups = [tree];
      }

      psdTools.collapseLayouts = groups;

      return groups.map(function (group) {
        var layers = (0, _flatLayers2.default)(group);

        return {
          title: group.name,
          layers: layers,
          top: group.top,
          width: psd.header.cols,
          height: group.height
        };
      });
    }).filter(function (layout) {
      return layout.layers.length;
    }).reduce(function (rest, next, index, length) {
      next.size = rest.size || 0;

      rest.size = (rest.size || 0) + next.layers.length;

      rest.push(next);
      if (index === length - 1) {
        rest.forEach(function (layout) {
          layout.allSize = rest.size;
        });
      }

      return rest;
    }, []).mapSeries(function (layout, inx) {
      return _bluebird2.default.mapSeries(layout.layers, function (layer) {
        return psdTools.parseLayer(layer.layer).then(function (elem) {
          var len = layout.allSize;
          if (!layout.size) {
            layout.size = 0;
          }
          layout.size += 1;

          dataProcesser({
            layer: layer,
            count: layout.size,
            length: len,
            index: inx
          });

          elem.top -= layout.top;
          layer.elem = elem;

          return layer;
        });
      }).then(function (layers) {
        layout.layers = layers.filter(function (layer) {
          return !layer.clippingMaskCached;
        });
        return layout;
      });
    }).filter(function (layout) {
      layout.layers.filter(function (layer) {
        return layer.elem;
      });

      return layout;
    }).then(function (layouts) {
      var parseGroup = function parseGroup (array, elem) {
        var elements = walkLayout(elem.children());

        if (psdTools.options.groupMode === 'tag' && elem.name.match(/\[\s*?(group)\s*?\]\s*(.*?)?\s*$/i) || psdTools.options.groupMode === 'merge') {
          var group = psdTools.createGroup(elements);

          array.push(group);
          return;
        }

        array.push.apply(array, _toConsumableArray(elements));
      };

      var parseLayer = function parseLayer (array, elem) {
        array.push(elem.maskElement || elem.elem || elem);
      };

      var walkLayout = function walkLayout (elems) {
        var array = [];
        elems.reverse().forEach(function (elem) {
          if (elem.visible() && !elem.clippingMaskCached) {
            if (elem.type === 'group' && elem.hasChildren()) {
              parseGroup(array, elem);
            } else if (elem.type === 'layer' && !elem.isEmpty()) {
              parseLayer(array, elem);
            }
          }
        });
        return array;
      };

      if (psdTools.options.groupMode === 'flat') {
        return layouts.map(function (layout) {
          var elems = layout.layers.reverse().map(function (layer) {
            if (layer.layer.isPreviousMask && layer.maskElement) {
              return layer.maskElement;
            }
            return layer.elem;
          });

          return {
            backgroundColor: '#FFFFFF',
            title: layout.title,
            height: layout.height,
            width: layout.width,
            elems: elems
          };
        });
      } else {
        return psdTools.collapseLayouts.map(function (layout) {
          var elems = walkLayout(layout.children());
          return {
            backgroundColor: '#FFFFFF',
            title: layout.name,
            height: layout.height,
            width: layout.width,
            elems: elems
          };
        });
      }
    });
  },
  parseLayer: function parseLayer (layer) {
    var getLayerType = function getLayerType (layer) {
      if (layer.clipped) {
        return 'mask';
      }

      if (layer.typeTool) {
        return 'text';
      }

      if (psdTools.options.parseSVG && (layer.vectorMask || layer.vectorStroke)) {
        return 'svg';
      }

      return 'image';
    };

    return _bluebird2.default.try(function () {
      if (layer.isPreviousMask) {
        return {};
      }
      var type = getLayerType(layer);
      var method = (0, _lodash2.default)('parse-' + type);

      if (!psdTools[method]) {
        throw new Error('No layer(' + type + ') parser defined');
      }

      return psdTools[method](layer, psdTools.options);
    }).delay(160).then(function (elem) {
      return elem;
    });
  },


  parseText: _bluebird2.default.method(function (layer, options) {
    var typeTool = layer.typeTool();
    var font = typeTool.export().font;
    var textData = typeTool.textData;
    var styles = typeTool.styles();

    var content = (0, _parseTextContent2.default)(typeTool.textValue, styles);

    var transform = (0, _parseTransform2.default)(typeTool.transform);
    var _transform = (0, _decomposeTransform2.default)(transform);

    var fontSize = (0, _getStyle2.default)(font.sizes, 12);

    if (_transform.scaleX > 0) {
      fontSize *= _transform.scaleY;
    }
    fontSize = +fontSize.toFixed(2);

    var textDecoration = 'none';
    if ((0, _getStyle2.default)(styles.Strikethrough)) {
      textDecoration = 'line-through';
    } else if ((0, _getStyle2.default)(styles.Underline)) {
      textDecoration = 'underline';
    }

    var letterSpacing = styles.Tracking ? styles.Tracking[0] : 0;


    var elem = (0, _standardLayerProps2.default)(layer);

    var textAlign = (0, _getStyle2.default)(font.alignment);

    elem.writingMode = textData.Ornt && textData.Ornt.value === 'Vrtc' ? 'vertical-rl' : 'horizontal-tb';

    var bounds = textData.bounds || { Rght: { value: 0 }, Left: { value: 0 }, Btom: { value: 0 }, 'Top ': { value: 0 } };

    // var height = (bounds.Btom.value - bounds['Top '].value) * _transform.scaleY || elem.height;
    var height = elem.height;
    var width = (bounds.Rght.value - bounds.Left.value) * _transform.scaleX || elem.width;
    var lineHeight = (0, _getStyle2.default)(styles.Leading, fontSize * 1);
    if (_transform.scaleX > 0) {
      lineHeight = (lineHeight * _transform.scaleY).toFixed(2) * 1;
    }
    if ((0, _getStyle2.default)(styles.AutoLeading)) {
      lineHeight = fontSize > height ? fontSize : height;
    }

    elem.multiLine = height >= fontSize * 2

    var left = elem.left;
    var top = elem.top;
    var topOffset = 0
    if (elem.multiLine) {
      topOffset = 0.5 * (lineHeight - fontSize);
    } else {
      topOffset = 0.5 * (lineHeight - height);
    }
    top -= topOffset;
    var leftOffset = (width - elem.width) / (content.length * 2)
    left -= leftOffset

    elem.color = (0, _stringifyRgba2.default)((0, _getStyle2.default)(font.colors));
    if (elem['Color Overlay']) {
      (0, _colorOverlay2.default)('text', elem, elem['Color Overlay'].color);
    }
    var fontFamily = typeTool.engineData.ResourceDict.FontSet
      .filter(f => !f.Synthetic)
      .map(f => f.Name)
    fontFamily = fontFamily[0] ? fontFamily[0] : (0, _getStyle2.default)(styles.names)

    var textShadow = {}
    if (layer.objectEffects) {
      var shadow = layer.objectEffects().data.DrSh
      if (shadow && shadow.enab) {
        textShadow.fill = `rgba(${shadow['Clr ']['Rd  ']},${shadow['Clr ']['Grn ']},${shadow['Clr ']['Bl  ']},${shadow.Opct.value / 100})`
        textShadow.blur = shadow.blur.value
        textShadow.degree = shadow.lagl.value
        textShadow.spread = shadow.Dstn.value
      }
    }
    return Object.assign(elem, {
      type: 'text',
      content: content,
      fontSize: fontSize,
      fontFamily: fontFamily,
      lineHeight: lineHeight,
      letterSpacing: letterSpacing,
      fontWeight: (0, _getStyle2.default)(styles.FauxBold) ? 700 : 400,
      fontStyle: (0, _getStyle2.default)(styles.FauxItalic) ? 'italic' : 'normal',
      textDecoration: textDecoration,
      textAlign: textAlign,
      textShadow: textShadow,
      height: height,
      width: width,
      top: top,
      left: left
    });
  }),

  parseImage: function parseImage (layer, options) {
    if (options.imageMaxWidth && layer.width > options.imageMaxWidth || options.imageMaxHeight && layer.height > options.imageMaxHeight) {
      throw new Error('timeout');
    }
    var elem = (0, _standardLayerProps2.default)(layer, options);
    var boxShadow = {}
    if (layer.objectEffects) {
      var shadow = layer.objectEffects().data.DrSh
      if (shadow && shadow.enab) {
        boxShadow.fill = `rgba(${shadow['Clr ']['Rd  ']},${shadow['Clr ']['Grn ']},${shadow['Clr ']['Bl  ']},${shadow.Opct.value / 100})`
        boxShadow.blur = shadow.blur.value
        boxShadow.degree = shadow.lagl.value
        boxShadow.spread = shadow.Dstn.value
      }
      var strokeData = layer.objectEffects().data.FrFX
      if (strokeData && strokeData.enab) {
        var strokeColor = 'rgb(' + Math.round(strokeData['Clr ']['Rd  ']) + ',' + Math.round(strokeData['Clr ']['Grn ']) + ',' + Math.round(strokeData['Clr ']['Bl  ']) + ')';
        var strokeWidth = strokeData['Sz  '].value
        elem.stroke = { width: strokeWidth, color: strokeColor }
      }
    }



    elem.boxShadow = boxShadow
    elem.blendingMode = layer.blendMode.mode
    elem.type = 'image';
    elem.name = layer.name
    if (!layer.image) {
      return elem;
    }
    elem.url = layer.image.toBase64();
    return elem
    // return (0, _imageToBase2.default)(layer.image, elem.$config).then(function (url) {
    //   elem.url = url;
    //   return new _bluebird2.default(function (resolve) {
    //     if (elem['Color Overlay']) {
    //       resolve((0, _colorOverlay2.default)('image', elem, elem['Color Overlay'].color));
    //     } else {
    //       resolve(elem);
    //     }
    //   });
    // });
  },


  parseMask: _bluebird2.default.method(function (layer, options) {
    var returnMask = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

    var elem = (0, _standardLayerProps2.default)(layer, options);
    elem.opacity = 1;

    var masks = [layer];
    var getClippingMaskCached = function getClippingMaskCached (item) {
      if (item.clippingMaskCached) {
        masks.unshift(item.layer);
        return getClippingMaskCached(item.clippingMaskCached);
      }
      return item;
    };
    var clippingMasks = getClippingMaskCached(layer.node.clippingMaskCached);

    var getClippingMasks = function getClippingMasks (elements) {
      var layers = [];

      elements.forEach(function (layer) {
        if (!layer.isRoot() && !layer.visible() || layer.isLayer() && layer.isEmpty() || layer.isGroup() && !layer.hasChildren()) { } else {
          if (layer.type === 'group') {
            layers.push.apply(layers, _toConsumableArray((0, _flatLayers2.default)(layer)));
          } else {
            layers.push(layer);
          }
        }
      });

      return _bluebird2.default.mapSeries(layers, function (clippingMask) {
        var maskElement = Object.assign({}, elem);
        maskElement.clip = {
          left: clippingMask.left - maskElement.left,
          top: clippingMask.top - maskElement.top,
          right: maskElement.left + maskElement.width - (clippingMask.left + clippingMask.width),
          bottom: maskElement.top + maskElement.height - (clippingMask.top + clippingMask.height)
        };

        maskElement.left = clippingMask.left;
        maskElement.top = clippingMask.top;
        maskElement.width = clippingMask.width;
        maskElement.height = clippingMask.height;

        var image = clippingMask.layer.image;
        return (0, _imageToBase2.default)(image, elem.$config).then(function (url) {
          maskElement.mask = url;
          clippingMask.maskElement = maskElement;

          return maskElement;
        });
      });
    };

    return (0, _mergeMasks2.default)(masks, elem).then(function (res) {
      elem.url = res;
      elem.type = 'mask';

      var maskGroups = clippingMasks.children().length > 0 ? clippingMasks.children() : [clippingMasks];

      return getClippingMasks(maskGroups).then(function () {
        return elem;
      });
    });
  }),

  parseSvg: _bluebird2.default.method(function (layer, options) {
    var elem = (0, _standardLayerProps2.default)(layer);
    elem.name = layer.name
    elem.type = 'svg';
    layer.elem = elem;
    var boxShadow = {}
    if (layer.objectEffects) {
      var shadow = layer.objectEffects().data.DrSh
      if (shadow && shadow.enab) {
        boxShadow.fill = `rgba(${shadow['Clr ']['Rd  ']},${shadow['Clr ']['Grn ']},${shadow['Clr ']['Bl  ']},${shadow.Opct.value / 100})`
        boxShadow.blur = shadow.blur.value
        boxShadow.degree = shadow.lagl.value
        boxShadow.spread = shadow.Dstn.value
      }
    }
    elem.boxShadow = boxShadow
    elem.blendingMode = layer.blendMode.mode

    return psdTools.parseSvgContent(layer).then(function (res) {
      elem.content = res.content;
      elem.colors = res.colors;
      elem.isLine = res.isLine
      delete layer.elem;

      return '';
    }).then(function (url) {
      elem.url = url;
      return elem;
    });
  }),

  parseSvgContent: function parseSvgContent (layer) {
    var svgColors = {};
    var isLine = false
    var createSvg = function createSvg (layer) {
      var res = (0, _createSvgPath2.default)(svgColors, layer);
      isLine = res.isLine
      return '<?xml version="1.0" encoding="utf-8" ?>\n                    <svg width="' + layer.width + '" height="' + layer.height + '" viewBox="0 0 ' + layer.width + ' ' + layer.height + '"\n                        xmlns="http://www.w3.org/2000/svg"\n                        xmlns:xlink="http://www.w3.org/1999/xlink">\n                                               ' + res.path + '\n                    </svg>';
    };
    var svgEle = {
      content: createSvg(layer),
      colors: svgColors,
      isLine: isLine
    };


    return _bluebird2.default.resolve(svgEle);
  },
  createGroup: function createGroup (elements) {
    var bbox = (0, _rect.getBBoxByElements)(elements);

    var group = {
      type: 'group',
      height: bbox.height,
      width: bbox.width,
      left: bbox.left,
      top: bbox.top,
      elems: []
    };

    elements.forEach(function (ele) {
      ele.top -= group.top;
      ele.left -= group.left;

      group.elems.push(ele);
    });

    return group;
  }
};

exports.default = psdTools;
module.exports = exports['default'];
