'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createPaths;

var _onecolor = require('onecolor');

var _onecolor2 = _interopRequireDefault(_onecolor);

var _colorOverlay = require('../utils/color-overlay');

var _colorOverlay2 = _interopRequireDefault(_colorOverlay);

function _interopRequireDefault (obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createPaths (svgColors, layer) {
  var points = layer.vectorMask().paths;

  var fill = 'none';
  if (layer.solidColor) {
    fill = (0, _onecolor2.default)('rgb(' + layer.solidColor().color().join(',') + ')').hex();
  }

  var strokeStr = '';
  if (layer.vectorStroke) {
    var strokeData = layer.vectorStroke().data;
    var stroke = void 0;
    if (1) {
      var _strokeColor = strokeData.strokeStyleContent['Clr '];
      var _ref = [Math.round(_strokeColor['Rd  ']), Math.round(_strokeColor['Grn ']), Math.round(_strokeColor['Bl  '])],
        r = _ref[0],
        g = _ref[1],
        b = _ref[2];

      stroke = (0, _onecolor2.default)('rgb(' + r + ',' + g + ',' + b + ')').hex();
    }

    var strokeWidth = void 0;
    if (1) {
      strokeWidth = strokeData.strokeStyleLineWidth;
      strokeWidth = strokeWidth.value + { 'Points': 'pt', 'Pixels': 'px' }[strokeWidth.unit];
    }

    var strokeLinecap = void 0;
    if (1) {
      strokeLinecap = {
        strokeStyleButtCap: 'butt',
        strokeStyleRoundCap: 'round',
        strokeStyleSquareCap: 'square'
      }[strokeData.strokeStyleLineCapType.value];
    }

    var strokeLinejoin = void 0;
    if (1) {
      strokeLinejoin = {
        'strokeStyleMiterJoin': 'miter',
        'strokeStyleRoundJoin': 'round',
        'strokeStyleBevelJoin': 'bevel'
      }[strokeData.strokeStyleLineJoinType.value];
    }

    var strokeDasharray = void 0;
    if (1) {
      var dashArray = strokeData.strokeStyleLineDashSet;
      var dashes = [];
      if (dashArray.length) {
        dashArray.forEach(function (dash) {
          dashes.push(dash.value * parseInt(strokeWidth, 10));
        });
      }
      strokeDasharray = dashes.join(' ');
    }

    var _key = (new Date().getTime() + Math.random() * 100000).toString(36);
    svgColors[_key] = stroke;

    var strokeColor = svgColors[_key];
    if (strokeData.fillEnabled) {
      var fillData = layer.vectorStrokeContent().data['Clr '];
      if (fillData) {
        var _ref2 = [Math.round(fillData['Rd  ']), Math.round(fillData['Grn ']), Math.round(fillData['Bl  '])],
          _r = _ref2[0],
          _g = _ref2[1],
          _b = _ref2[2];

        fill = (0, _onecolor2.default)('rgb(' + _r + ',' + _g + ',' + _b + ')').hex();
      }
    } else {
      fill = 'none';

      if (layer.height < 10) {
        strokeStr = '\n         shape-rendering="crispEdges"    stroke=\'currentColor\'\n            stroke-width=\'' + (layer.height - strokeData.strokeStyleLineWidth.value + 1) + '\'\n            stroke-linecap=\'' + strokeLinecap + '\'\n            stroke-linejoin=\'' + strokeLinejoin + '\'\n            stroke-dasharray=\'' + strokeDasharray + '\'\n        ';
        return { path: `<line x1="0" y1="1" x2="${layer.width}" y2="1" ${strokeStr}/>`, isLine: true }
      } else if (layer.width < 10) {
        strokeStr = '\n     shape-rendering="crispEdges"        stroke=\'currentColor\'\n            stroke-width=\'' + (layer.width - strokeData.strokeStyleLineWidth.value + 1) + '\'\n            stroke-linecap=\'' + strokeLinecap + '\'\n            stroke-linejoin=\'' + strokeLinejoin + '\'\n            stroke-dasharray=\'' + strokeDasharray + '\'\n        ';
        return { path: `<line x1="1" y1="0" x2="1" y2="${layer.height}" ${strokeStr}/>`, isLine: true }
      }
    }

    strokeStr = '\n            stroke=\'' + strokeColor + '\'\n            stroke-width=\'' + strokeWidth + '\'\n            stroke-linecap=\'' + strokeLinecap + '\'\n            stroke-linejoin=\'' + strokeLinejoin + '\'\n            stroke-dasharray=\'' + strokeDasharray + '\'\n        ';


  }

  if (layer.objectEffects) {
    var strokeData = layer.objectEffects().data.FrFX
    if (strokeData) {
      var strokeColor = 'rgb(' + Math.round(strokeData['Clr ']['Rd  ']) + ',' + Math.round(strokeData['Clr ']['Grn ']) + ',' + Math.round(strokeData['Clr ']['Bl  ']) + ')';
      var strokeWidth = strokeData['Sz  '].value
      strokeStr = '\n            stroke=\'' + strokeColor + '\'\n            stroke-width=\'' + strokeWidth + '\'\n        ';
    }

  }

  var d = [];
  if (1) {
    var startPoint = 2;

    var parsePoint = function parsePoint (points) {
      return points.map(function (point) {
        var canvas_w = layer.header.width,
          canvas_h = layer.header.height;
        var left = layer.left,
          top = layer.top;

        ['precedingHoriz', 'precedingVert', 'anchorHoriz', 'anchorVert', 'leavingHoriz', 'leavingVert'].forEach(function (key, index) {
          var num = point[key];

          if (num > 200) {
            num = num - 256;
          }

          if (index % 2 === 0) {
            point[key] = num * canvas_w - left;
          } else {
            point[key] = num * canvas_h - top;
          }
        });

        return point;
      });
    };

    while (startPoint < points.length) {
      var desc = points[startPoint];

      var closePath = desc.recordType === 0;
      var endPoint = startPoint + desc.numPoints;

      var snippet = parsePoint(points.slice(startPoint + 1, endPoint + 1));

      var i = 0,
        len = snippet.length,
        str = void 0;
      while (i < len - 1) {
        var point = snippet[i],
          nextPoint = snippet[++i];
        if (!str) {
          str = 'M' + point.anchorHoriz + ' ' + point.anchorVert;
        }
        str += 'C' + point.leavingHoriz + ' ' + point.leavingVert + '\n                    ' + nextPoint.precedingHoriz + ' ' + nextPoint.precedingVert + '\n                    ' + nextPoint.anchorHoriz + ' ' + nextPoint.anchorVert;
      }

      if (closePath) {
        var _point = snippet[snippet.length - 1];
        var _nextPoint = snippet[0];

        str += 'C' + _point.leavingHoriz + ' ' + _point.leavingVert + '\n                    ' + _nextPoint.precedingHoriz + ' ' + _nextPoint.precedingVert + '\n                    ' + _nextPoint.anchorHoriz + ' ' + _nextPoint.anchorVert + ' Z';
      }

      startPoint = endPoint + 1;

      d.push(str);
    }
  }

  var key = new Date().getTime();

  if (layer.elem['Color Overlay']) {
    (0, _colorOverlay2.default)('svg', layer.elem, layer.elem['Color Overlay'].color, fill);
    fill = layer.elem.color;
  }
  svgColors[key] = fill;
  d = d.join(' ');
  return { path: '<path d=\'' + d + '\' ' + strokeStr + ' fill=\'{{colors["' + key + '"]}}\' fill-rule=\'evenodd\' />', isLine: false };
}
module.exports = exports['default'];
