'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = parseEffects;

var _shadow = require('../utils/shadow');

var _shadow2 = _interopRequireDefault(_shadow);

var _stroke = require('../utils/stroke');

var _stroke2 = _interopRequireDefault(_stroke);

var _colorOverlay = require('../utils/color-overlay');

var _colorOverlay2 = _interopRequireDefault(_colorOverlay);

var _getColor = require('../parse-tools/get-color');

var _getColor2 = _interopRequireDefault(_getColor);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parseEffects(layer) {
    var effectsNameMap = {
        DrSh: 'boxShadow',

        FrFX: 'boxShadow',

        ebbl: 'Bevel and Emboss',

        IrSh: 'boxShadow',

        IrGl: 'Inner Glow',

        ChFX: 'Satin',

        SoFi: 'Color Overlay',

        GrFl: 'Gradient Overlay',

        patternFill: 'Pattern Overlay',

        OrGl: 'Outer Glow'
    };

    var effectsParsers = {
        DrSh: _shadow2.default,

        FrFX: _stroke2.default,

        IrSh: _shadow2.default,

        SoFi: _getColor2.default
    };

    var effects = {};
    var effectsData = layer.objectEffects ? layer.objectEffects().data : null;

    if (!effectsData || !effectsData.masterFXSwitch) {
        return effects;
    }

    Object.keys(effectsData).forEach(function (key) {
        var data = effectsData[key];

        if (!data || !data.enab) {
            return;
        }

        var name = effectsNameMap[key];
        var parser = effectsParsers[key];
        var effect = parser && parser(data);

        if (key === 'IrSh') {
            effect.inset = true;
        }

        if (effect) {
            effects[name] = effect;
        }
    });

    return effects;
}
module.exports = exports['default'];