'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function (image, options) {
    var canvas = drawImageToCanvas(image);

    if (typeof options === 'number') {
        options = {
            quality: options
        };
    }

    options = (0, _lodash2.default)(options, {
        format: canvas.hasAlphaTunnel ? 'png' : 'jpg',
        quality: 60
    });

    var format = options.format;
    if (format === 'jpg') {
        format = 'jpeg';
    }

    var quality = options.quality / 100;
    var url = canvas.toDataURL('image/' + format, quality);

    return _bluebird2.default.resolve(url);
};

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _lodash = require('lodash.defaults');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var drawImageToCanvas = function drawImageToCanvas(image) {
    var canvas = document.createElement('canvas');
    var canvasCtx = canvas.getContext('2d');

    var width = image.width();
    var height = image.height();

    var imgData = canvasCtx.createImageData(width, height);

    var hasAlphaTunnel = false;
    var imgDataArr = imgData.data;
    var pixelData = image.pixelData;

    canvas.width = width;
    canvas.height = height;

    var len = pixelData.length;
    var per = 8;
    var remainder = len % per;

    for (var i = len; i >= per; i -= per) {
        imgDataArr[i - 1] = pixelData[i - 1];
        imgDataArr[i - 2] = pixelData[i - 2];
        imgDataArr[i - 3] = pixelData[i - 3];
        imgDataArr[i - 4] = pixelData[i - 4];

        imgDataArr[i - 5] = pixelData[i - 5];
        imgDataArr[i - 6] = pixelData[i - 6];
        imgDataArr[i - 7] = pixelData[i - 7];
        imgDataArr[i - 8] = pixelData[i - 8];

        if (!hasAlphaTunnel && (imgDataArr[i - 1] < 255 || imgDataArr[i - 5] < 255)) {
            hasAlphaTunnel = true;
        }
    }
    while (remainder >= 0) {
        imgDataArr[remainder] = pixelData[remainder];
        --remainder;
    };

    canvas.hasAlphaTunnel = hasAlphaTunnel;

    canvasCtx.putImageData(imgData, 0, 0);

    return canvas;
};

;
module.exports = exports['default'];