'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _onecolor = require('onecolor');

var _onecolor2 = _interopRequireDefault(_onecolor);

var _getGradientColor = require('../utils/get-gradient-color');

var _getGradientColor2 = _interopRequireDefault(_getGradientColor);

var _mergeColors = require('../utils/merge-colors');

var _mergeColors2 = _interopRequireDefault(_mergeColors);

function _interopRequireDefault (obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _colors = [];

var rndHex = function rndHex () {
  return Math.floor(Math.random() * 0xffffff).toString(16);
};

var formatColor = function formatColor (inp) {
  if (!inp || typeof inp === 'string') {
    return (0, _onecolor2.default)(inp || '#000').hex();
  }

  var data = {
    r: 0,
    g: 0,
    b: 0,
    a: 1
  };

  if (Array.isArray(inp)) {
    data.r = inp[0] || 0;
    data.g = inp[1] || 0;
    data.b = inp[2] || 0;
  } else {
    data.r = Math.round(inp['Rd  '] || inp.r || 0);
    data.g = Math.round(inp['Grn '] || inp.g || 0);
    data.b = Math.round(inp['Bl  '] || inp.b || 0);
  }

  return (0, _onecolor2.default)([data.r, data.g, data.b, isFinite(data.a) ? data.a : 1]).hex();
};

var parsePoints = function parsePoints (layer, points) {
  var types = ['precedingHoriz', 'precedingVert', 'anchorHoriz', 'anchorVert', 'leavingHoriz', 'leavingVert'];

  return points.map(function (point) {
    var canvasW = layer.header.width;
    var canvasH = layer.header.height;
    var left = layer.left;
    var top = layer.top;

    types.forEach(function (key, index) {
      var num = point[key];

      if (num > 200) {
        num = num - 256;
      }

      if (index % 2 === 0) {
        point[key] = num * canvasW - left;
      } else {
        point[key] = num * canvasH - top;
      }
    });

    return point;
  });
};

var parsePaths = function parsePaths (layer) {
  var points = layer.vectorMask().paths;
  var pointsLen = points.length;
  var ret = [];

  var startPoint = 2;

  while (startPoint < pointsLen) {
    var desc = points[startPoint];
    var closePath = desc.recordType === 0;
    var endPoint = startPoint + desc.numPoints;
    var snippet = parsePoints(layer, points.slice(startPoint + 1, endPoint + 1));

    var i = 0;
    var len = snippet.length;
    var str = void 0;

    while (i < len - 1) {
      var point = snippet[i];
      var nextPoint = snippet[++i];

      if (!str) {
        str = 'M' + point.anchorHoriz + ' ' + point.anchorVert;
      }

      str += 'C' + point.leavingHoriz + ' ' + point.leavingVert + '\n                ' + nextPoint.precedingHoriz + ' ' + nextPoint.precedingVert + '\n                ' + nextPoint.anchorHoriz + ' ' + nextPoint.anchorVert;
    }

    if (closePath) {
      var _point = snippet[snippet.length - 1];
      var _nextPoint = snippet[0];

      str += 'C' + _point.leavingHoriz + ' ' + _point.leavingVert + '\n                ' + _nextPoint.precedingHoriz + ' ' + _nextPoint.precedingVert + '\n                ' + _nextPoint.anchorHoriz + ' ' + _nextPoint.anchorVert + ' Z';
    }

    startPoint = endPoint + 1;

    ret.push(str);
  }

  return ret;
};

var addColor = function addColor (color) {
  _colors.push(color);

  return '{{colors[' + (_colors.length - 1) + ']}}';
};

var getDocumentSize = function getDocumentSize (layer) {
  return {
    documentWidth: layer.header.cols,
    documentHeight: layer.header.rows
  };
};

var calGradientCoordination = function calGradientCoordination (canvasWidth, canvasHeight, degree) {
  // 角度为0：从左向右渐变
  // 角度为90：从下往上渐变
  // 角度为180：从右向左渐变
  // 角度-90：从上往下渐变
  // 角度为0-90度之间：
  let start, end
  end = getCoordination(canvasWidth, canvasHeight, degree)
  let oppositionDegree = 0 // 对立角度
  if (degree >= 0) {
    oppositionDegree = degree - 180
  } else {
    oppositionDegree = 180 + degree
  }
  start = getCoordination(canvasWidth, canvasHeight, oppositionDegree)
  return {
    start,
    end
  }
}

var getCoordination = function getCoordination (width, height, degree) {
  let coordination = {} // 坐标点
  let diffDegree = 0
  // 矩形的四个点相对于中心店的角度
  const rectAngles = calRectAngle(width, height)
  const x0 = width / 2
  const y0 = height / 2
  // 计算角度差
  if (degree >= 0 && degree < 90) {
    diffDegree = Math.abs(degree - rectAngles.rtd)
  } else if (degree >= 90 && degree <= 180) {
    diffDegree = Math.abs(degree - rectAngles.ltd)
  } else if (degree < 0 && degree >= -90) {
    diffDegree = Math.abs(degree - rectAngles.rbd)
  } else if (degree < -90 && degree >= -180) {
    diffDegree = Math.abs(degree - rectAngles.lbd)
  }
  if (degree >= 0 && degree <= 90) {
    coordination = {
      x:
        x0 +
        Math.round(
          Math.cos((degree * Math.PI) / 180) *
          Math.cos((diffDegree * Math.PI) / 180) *
          Math.sqrt(y0 * y0 + x0 * x0)
        ),
      y:
        y0 -
        Math.round(
          Math.sin((degree * Math.PI) / 180) *
          Math.cos((diffDegree * Math.PI) / 180) *
          Math.sqrt(y0 * y0 + x0 * x0)
        )
    }
  } else if (degree > 90 && degree <= 180) {
    coordination = {
      x:
        x0 +
        Math.round(
          Math.cos((degree * Math.PI) / 180) *
          Math.cos((diffDegree * Math.PI) / 180) *
          Math.sqrt(y0 * y0 + x0 * x0)
        ),
      y:
        y0 -
        Math.round(
          Math.sin((degree * Math.PI) / 180) *
          Math.cos((diffDegree * Math.PI) / 180) *
          Math.sqrt(y0 * y0 + x0 * x0)
        )
    }
  } else if (degree < 0 && degree >= -90) {
    coordination = {
      x:
        x0 +
        Math.round(
          Math.cos((degree * Math.PI) / 180) *
          Math.cos((diffDegree * Math.PI) / 180) *
          Math.sqrt(y0 * y0 + x0 * x0)
        ),
      y:
        y0 -
        Math.round(
          Math.sin((degree * Math.PI) / 180) *
          Math.cos((diffDegree * Math.PI) / 180) *
          Math.sqrt(y0 * y0 + x0 * x0)
        )
    }
  } else if (degree >= -180 && degree < -90) {
    coordination = {
      x:
        x0 +
        Math.round(
          Math.cos((degree * Math.PI) / 180) *
          Math.cos((diffDegree * Math.PI) / 180) *
          Math.sqrt(y0 * y0 + x0 * x0)
        ),
      y:
        y0 -
        Math.round(
          Math.sin((degree * Math.PI) / 180) *
          Math.sin((diffDegree * Math.PI) / 180) *
          Math.sqrt(y0 * y0 + x0 * x0)
        )
    }
  }
  return coordination
}

var calRectAngle = function calRectAngle (x, y) {
  // 右上角的角度
  // console.log('右上角：', (Math.atan2(x, -y) * 180 / Math.PI - 90))
  // console.log('右下角：', (Math.atan2(x, y) * 180 / Math.PI - 90))
  // console.log('左上角：', (Math.atan2(-x, -y) * 180 / Math.PI - 90) + 360)
  // console.log('左下角：', (Math.atan2(-x, y) * 180 / Math.PI - 90))
  const rtd = (Math.atan2(x, -y) * 180) / Math.PI - 90
  const rbd = (Math.atan2(x, y) * 180) / Math.PI - 90
  const ltd = (Math.atan2(-x, -y) * 180) / Math.PI - 90 + 360
  const lbd = (Math.atan2(-x, y) * 180) / Math.PI - 90
  return {
    rtd,
    rbd,
    ltd,
    lbd
  }
}

var parseLinearGradient = function parseLinearGradient (gradientFillData, layer) {
  var x1 = 0;
  var y1 = 0;
  var x2 = x1 + layer.width;
  var y2 = 0;
  var width = layer.width;
  var height = layer.height;

  if (gradientFillData.Algn !== undefined && gradientFillData.Algn === false) {
    var _getDocumentSize = getDocumentSize(layer),
      documentWidth = _getDocumentSize.documentWidth,
      documentHeight = _getDocumentSize.documentHeight;

    x1 = -layer.left;
    x2 = documentWidth + x1;
    y1 = -layer.top;
    y2 = -layer.top;
    width = documentWidth;
    height = documentHeight;
  }
  if (gradientFillData.Angl) {
    var angle = gradientFillData.Angl.value;
    if (angle === 90) {
      x2 = x1;
      y2 = y1;
      y1 += height;
    } else if (angle === 270) {
      x2 = x1;
      y2 += height;
    } else {
      var coefficient = Math.tan(Math.PI * angle / 180);
      var cx = x1 + width / 2;
      var cy = y1 + height / 2;
      var constantValue = cy + coefficient * cx;
      var formulaCoordinate = function formulaCoordinate (_ref) {
        var x = _ref.x,
          y = _ref.y;

        if (x !== undefined && y === undefined) {
          return {
            x: x,
            y: -coefficient * x + constantValue
          };
        }
        if (y !== undefined && x === undefined) {
          return {
            y: y,
            x: (constantValue - y) / coefficient
          };
        }
      };

      var startX1 = x1;
      var startY1 = formulaCoordinate({ x: startX1 }).y;

      var startY2 = y1;
      var startX2 = formulaCoordinate({ y: startY2 }).x;

      x1 = startX2 < x1 ? startX2 : startX1;
      y1 = startX2 < x1 ? startY2 : startY1;

      var endX1 = x2;
      var endY1 = formulaCoordinate({ x: endX1 }).y;

      var endY2 = y2 + height;
      var endX2 = formulaCoordinate({ y: endY2 }).x;

      x2 = endX2 > x2 ? endX2 : endX1;
      y2 = endX2 > x2 ? endY2 : endY1;
    }
  };

  return 'x1="' + x1 + '" y1="' + y1 + '" x2="' + x2 + '" y2="' + y2 + '"';
};

var parseRadialGradient = function parseRadialGradient (gradientFillData, layer) {
  var width = layer.width;
  var height = layer.height;
  var cx = width / 2;
  var cy = height / 2;
  var radius = width / 2;

  if (gradientFillData.Algn !== undefined && gradientFillData.Algn === false) {
    var _getDocumentSize2 = getDocumentSize(layer),
      documentWidth = _getDocumentSize2.documentWidth,
      documentHeight = _getDocumentSize2.documentHeight;

    width = documentWidth;
    height = documentHeight;

    cx = -layer.left + width / 2;
    cy = -layer.top + height / 2;
    radius = width / 2;
  }

  if (gradientFillData.Angl) {
    var angle = gradientFillData.Angl.value;
    if (angle !== 180 && angle !== 360) {
      radius = height / 2 / Math.abs(Math.sin(Math.PI * angle / 180));
    }
  }

  return 'cx="' + cx + '" cy="' + cy + '" r="' + radius + '"';
};

var parseGradient = function parseGradient (gradientData, layer) {
  var type = gradientData.Type.value === 'Lnr ' ? 'linearGradient' : 'radialGradient';

  var stops = (0, _getGradientColor2.default)(gradientData);

  var stopsStr = stops.map(function (item) {
    var offsetStr = (100 * item.offset).toFixed(2).replace(/\.?0+$/, '');

    return '<stop offset="' + offsetStr + '%" stop-color="' + item.color + '" />';
  }).join('');

  var id = 'gr_' + rndHex();
  var positionStr = type === 'linearGradient' ? parseLinearGradient(gradientData, layer) : parseRadialGradient(gradientData, layer);

  var content = '<defs><' + type + ' id="' + id + '" ' + positionStr + ' gradientUnits="userSpaceOnUse">' + stopsStr + '</' + type + '></defs>';

  return {
    content: content,
    id: id
  };
};

var parseFill = function parseFill (layer, strokeFillEnabled) {
  var fillData = strokeFillEnabled && layer.vectorStrokeContent ? layer.vectorStrokeContent().data : layer.gradientFill ? layer.gradientFill().data : layer.solidColor ? {
    'Clr ': layer.solidColor().color()
  } : {
      fillColor: '#00000000'
    };
  if (fillData['Clr ']) {
    var fillColor = formatColor(fillData['Clr ']);
    var overlays = layer.elem._effects['Color Overlay'];
    if (overlays) {
      overlays.forEach(function (item) {
        if (fillColor) {
          fillColor = (0, _mergeColors2.default)(fillColor, item.color);
        }
      });
    }
    return {
      fillColor: addColor(fillColor)
    };
  }
  if (fillData.Grad) {
    var _parseGradient = parseGradient(fillData, layer),
      content = _parseGradient.content,
      id = _parseGradient.id;

    var fillPlace = 'url(#' + id + ')';

    return {
      fillPlace: fillPlace,
      content: content
    };
  }

  return fillData;
};

var parseStroke = function parseStroke (layer) {
  var ret = {
    htmlAttrs: '',
    stroke: null,
    strokeWidth: 0,
    strokeLinecap: null,
    strokeLinejoin: null,
    strokeDashArray: null,
    fillEnabled: false,
    fillRule: null,
    content: ''
  };

  ret.fillRule = 'evenodd';

  if (!layer.vectorStroke) {
    var strokes = layer.elem._effects['stroke'];
    if (strokes) {
      ret.strokeWidth = strokes[0].width;
      ret.strokeAlignment = strokes[0].type === 'outer' ? 'outside' : strokes[0].type;
      ret.stroke = addColor(strokes[0].color);
    }

    return ret;
  }

  var strokeData = layer.vectorStroke().data;

  var strokeStyleContent = strokeData.strokeStyleContent;
  if (strokeStyleContent['Clr ']) {
    ret.stroke = addColor(formatColor(strokeStyleContent['Clr ']));
  }

  if (strokeStyleContent.Grad) {
    var gradientStroke = parseGradient(strokeStyleContent, layer);
    ret.content = gradientStroke.content;
    ret.stroke = 'url(#' + gradientStroke.id + ')';
  }

  var strokeWidthData = strokeData.strokeStyleLineWidth;
  if (strokeWidthData) {
    var strokeWidth = strokeWidthData.value;
    if (strokeWidthData.unit === 'Points') {
      strokeWidth = strokeWidth * layer.resolution / 72;
    }

    ret.strokeWidth = strokeWidth;
  }

  var strokeLinecapsMap = {
    strokeStyleButtCap: 'butt',
    strokeStyleRoundCap: 'round',
    strokeStyleSquareCap: 'square'
  };

  ret.strokeLinecap = strokeLinecapsMap[strokeData.strokeStyleLineCapType.value];

  var strokeLinejoinsMap = {
    'strokeStyleMiterJoin': 'miter',
    'strokeStyleRoundJoin': 'round',
    'strokeStyleBevelJoin': 'bevel'
  };

  ret.strokeLinejoin = strokeLinejoinsMap[strokeData.strokeStyleLineJoinType.value];

  var dashArray = strokeData.strokeStyleLineDashSet;
  if (dashArray && dashArray.length) {
    var _strokeWidth = parseInt(ret.strokeWidth, 10);

    ret.strokeDashArray = dashArray.map(function (dash) {
      return dash.value * _strokeWidth;
    });
  }

  var strokeLineAlignmentsMap = {
    strokeStyleAlignOutside: 'outside',
    strokeStyleAlignCenter: 'center',
    strokeStyleAlignInside: 'inside'
  };
  ret.strokeAlignment = strokeLineAlignmentsMap[strokeData.strokeStyleLineAlignment.value];

  ret.fillEnabled = strokeData.fillEnabled;

  return ret;
};

function createPaths (layer) {
  var colors = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  _colors = colors;
  var paths = parsePaths(layer);
  var strokeData = parseStroke(layer);
  var fillData = parseFill(layer, strokeData.fillEnabled);

  var pathAttrs = [fillData.fillPlace ? 'fill="' + fillData.fillPlace + '"' : 'fill="' + fillData.fillColor + '"', strokeData.fillRule ? 'fill-rule="' + strokeData.fillRule + '"' : '', strokeData.stroke ? 'stroke="' + strokeData.stroke + '"' : '', strokeData.strokeWidth ? 'stroke-width="' + strokeData.strokeWidth + '"' : '', strokeData.strokeLinecap ? 'stroke-linecap="' + strokeData.strokeLinecap + '"' : '', strokeData.strokeLinejoin ? 'stroke-linejoin="' + strokeData.strokeLinejoin + '"' : '', strokeData.strokeDashArray ? 'stroke-dasharray="' + strokeData.strokeDashArray.join(', ') + '"' : '', 'd="' + paths.join(' ').replace(/\s\s+/g, ' ') + '"'].filter(function (str) {
    return !!str;
  });

  var path = '<path ' + pathAttrs.join(' ') + ' />';

  var strokeAlignment = strokeData.strokeAlignment;
  var strokeWidth = strokeData.strokeWidth;

  var viewBox = '0 0 ' + layer.width + ' ' + layer.height;

  if (strokeAlignment === 'inside') {
    viewBox = 0 - strokeWidth / 2 + ' ' + (0 - strokeWidth / 2) + ' ' + (layer.width + strokeWidth) + ' ' + (layer.height + strokeWidth);
  } else if (strokeAlignment === 'outside') {
    viewBox = 0 + strokeWidth / 2 + ' ' + (0 + strokeWidth / 2) + ' ' + (layer.width - strokeWidth) + ' ' + (layer.height - strokeWidth);
  }

  var content = '<svg xmlns="http://www.w3.org/2000/svg" preserveAspectRatio="none" viewBox="' + viewBox + '">\n        <title>' + layer.name + '</title>\n        ' + (fillData.content || strokeData.content ? '<defs>' + fillData.content + strokeData.content + '</defs>' : '') + '\n        ' + path + '\n    </svg>';

  return {
    content: content,
    colors: _colors
  };
};

exports.default = parseGradient;
module.exports = exports['default'];
