"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = decomposeTransform;
function decomposeTransform(transform) {
    var pi = Math.PI;
    var acos = Math.acos;
    var atan = Math.atan;
    var sqrt = Math.sqrt;

    var a = transform.a;
    var b = transform.b;
    var c = transform.c;
    var d = transform.d;
    var rotation = 0;
    var skew = { x: 0, y: 0 };
    var scale = { x: 1, y: 1 };

    var determ = a * d - b * c;

    if (a || b) {
        var r = sqrt(a * a + b * b);

        rotation = b > 0 ? acos(a / r) : -acos(a / r);

        skew.x = atan((a * c + b * d) / (r * r));
        scale = { x: r, y: determ / r };
    } else if (c || d) {
        var s = sqrt(c * c + d * d);

        rotation = pi * 0.5 - (d > 0 ? acos(-c / s) : -acos(c / s));

        scale = { x: determ / s, y: s };
        skew.y = atan((a * c + b * d) / (s * s));
    } else {
            scale = { x: 0, y: 0 };
        }

    return {
        rotation: rotation || 0,
        scaleX: scale.x,
        scaleY: scale.y,
        skewX: skew.x,
        skewY: skew.y,
        positionX: transform.tx,
        positionY: transform.ty,
        matrix: transform
    };
}
module.exports = exports["default"];