'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = parseEffects;

var _effects = require('./effects');

var _effects2 = _interopRequireDefault(_effects);

var _effectParser = require('./effect-parser');

var _effectParser2 = _interopRequireDefault(_effectParser);

function _interopRequireDefault (obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parseEffects (layer) {
  var effects = {};
  var effectsData = layer.objectEffects ? layer.objectEffects().data : null;

  if (!effectsData || !effectsData.masterFXSwitch) {
    return effects;
  }

  var effectList = [];

  Object.keys(effectsData).forEach(function (key) {
    if (key === 'frameFXMulti') {
      var datas = effectsData[key];
      datas.forEach(function (data) {
        if (data.enab) {
          effectList.push({
            type: 'FrFX',
            data: data
          });
        }
      });
    }

    var data = effectsData[key];

    if (!data || !data.enab) {
      return;
    }

    effectList.push({
      type: key,
      data: data
    });
  });

  effectList.forEach(function (effectItem) {
    var name = _effects2.default[effectItem.type];
    var parser = _effectParser2.default[effectItem.type];
    var effect = parser && parser(effectItem.data);

    if (effect) {
      effects[name] = effects[name] || [];
      effects[name].push(effect);
    }
  });

  return effects;
}
module.exports = exports['default'];
