'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = patternGradientOverlay;

var _formatColor = require('../utils/format-color');

var _formatColor2 = _interopRequireDefault(_formatColor);

var _gradientColorTools = require('../utils/gradient-color-tools');

function _interopRequireDefault (obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getAlphaList = function getAlphaList (option) {
  var opacityPoints = option.opacityPoints,
    numLen = option.numLen;


  var alphaList = [];
  opacityPoints.forEach(function (item, inx) {
    var offset = +(item.Lctn / numLen).toFixed(2);
    var opacity = item.Opct.value / 100;

    if (item.Mdpn !== 50 && inx > 0) {
      var previousPoint = alphaList[alphaList.length - 1];
      var centerOffset = +(item.Mdpn / 100).toFixed(2);
      var mediumAlpha = (0, _gradientColorTools.getMediumAlpha)({
        startAlpha: previousPoint.color.a,
        endAlpha: opacity,
        offset: 0.5
      });

      alphaList.push({
        color: {
          r: null,
          g: null,
          b: null,
          a: mediumAlpha
        },
        offset: (offset - previousPoint.offset) * centerOffset + previousPoint.offset
      });
    }

    alphaList.push({
      color: {
        r: null,
        g: null,
        b: null,
        a: opacity
      },
      offset: offset
    });
  });

  if (alphaList[0].offset > 0) {
    var startAlpha = {
      color: (0, _extends3.default)({}, alphaList[0].color),
      offset: 0
    };
    alphaList.unshift(startAlpha);
  }

  if (alphaList[alphaList.length - 1].offset < 1) {
    var endAlpha = {
      color: (0, _extends3.default)({}, alphaList[alphaList.length - 1].color),
      offset: 1
    };
    alphaList.push(endAlpha);
  }

  return alphaList;
};

var getColorListWithScale = function getColorListWithScale (option) {
  var colorList = option.colorList,
    numLen = option.numLen,
    scale = option.scale;


  var newNumLen = numLen * scale;

  if (scale < 1) {
    var startIndex = numLen / 2 * (1 - scale);
    colorList.forEach(function (item) {
      var offset = (item.offset * newNumLen + startIndex) / numLen;
      item.offset = offset;
    });

    return colorList;
  }

  if (scale > 1) {
    var newColorList = [];

    var expandStartIndex = numLen / 2 * (scale - 1);
    var expandStartOffset = expandStartIndex / newNumLen;
    var expandEndIndex = expandStartIndex + numLen;
    var expandEndOffset = expandEndIndex / newNumLen;

    var startPointColor = (0, _gradientColorTools.getTargetGradientColor)({
      colorList: colorList,
      offset: expandStartOffset
    }).color;

    newColorList.push({
      color: startPointColor,
      offset: 0
    });

    colorList.forEach(function (item) {
      var offset = item.offset * newNumLen;
      if (offset > expandStartIndex && offset < expandEndIndex) {
        newColorList.push({
          color: item.color,
          offset: (offset - expandStartIndex) / numLen
        });
      }
    });

    var endPointColor = (0, _gradientColorTools.getTargetGradientColor)({
      colorList: colorList,
      offset: expandEndOffset
    }).color;

    newColorList.push({
      color: endPointColor,
      offset: 1
    });

    return newColorList;
  }

  return colorList;
};

var getColorListWithOpacity = function getColorListWithOpacity (option) {
  var colorList = option.colorList,
    opacity = option.opacity;


  var newColorList = colorList.map(function (item) {
    item.color.a *= opacity;

    return item;
  });

  return newColorList;
};

var getColorList = function getColorList (option) {
  var colorPoints = option.colorPoints,
    numLen = option.numLen;


  var colorList = [];
  colorPoints.map(function (item, inx) {
    var offset = +(item.Lctn / numLen).toFixed(2);
    var color = item['Clr '];
    var nColor = {
      r: Math.round(color['Rd  ']),
      g: Math.round(color['Grn ']),
      b: Math.round(color['Bl  ']),
      a: null
    };

    if (item.Mdpn !== 50 && inx > 0) {
      var previousPoint = colorList[colorList.length - 1];
      var centerOffset = +(item.Mdpn / 100).toFixed(2);
      var mediumColor = (0, _gradientColorTools.getMediumColor)({
        startColor: previousPoint.color,
        endColor: nColor,
        offset: 0.5,
        hasAlpha: false
      });

      colorList.push({
        color: mediumColor,
        offset: (offset - previousPoint.offset) * centerOffset + previousPoint.offset
      });
    }

    colorList.push({
      color: nColor,
      offset: offset
    });
  });

  if (colorList[0].offset > 0) {
    var startColor = {
      color: (0, _extends3.default)({}, colorList[0].color),
      offset: 0
    };
    colorList.unshift(startColor);
  }

  if (colorList[colorList.length - 1].offset < 1) {
    var endColor = {
      color: (0, _extends3.default)({}, colorList[colorList.length - 1].color),
      offset: 1
    };
    colorList.push(endColor);
  }

  return colorList;
};

function patternGradientOverlay (data) {
  var styleData = data['Grad'];
  var numLen = +styleData.Intr || 4096;
  var colorList = getColorList({
    colorPoints: styleData.Clrs,
    numLen: numLen
  });

  var alphaList = getAlphaList({
    opacityPoints: styleData['Trns'],
    numLen: numLen
  });

  var newColorList = (0, _gradientColorTools.mergeColorAndAlpha)(colorList, alphaList);

  if (data['Scl ']) {
    var scale = data['Scl '].value / 100;
    newColorList = getColorListWithScale({
      colorList: newColorList,
      scale: scale,
      numLen: numLen
    });
  }

  if (data['Opct']) {
    var opacity = data['Opct'].value / 100;
    newColorList = getColorListWithOpacity({
      colorList: newColorList,
      opacity: opacity
    });
  }

  var stops = newColorList.map(function (item) {
    return {
      color: (0, _formatColor2.default)(item.color, 'cssa'),
      offset: +item.offset.toFixed(2)
    };
  });

  if (data.Rvrs) {
    stops = stops.reverse();
    stops.forEach(function (stop) {
      stop.offset = 1 - stop.offset;
    });
  }

  if (stops.length > 2 && stops[0].color === stops[1].color) {
    stops.shift();
  }

  if (stops.length > 2 && stops[stops.length - 1].color === stops[stops.length - 2].color) {
    stops.pop();
  }

  return stops;
};
module.exports = exports['default'];
