'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getMediumColor = getMediumColor;
exports.getMediumAlpha = getMediumAlpha;
exports.getTargetGradientColor = getTargetGradientColor;
exports.completeRGBA = completeRGBA;
exports.mergeColorAndAlpha = mergeColorAndAlpha;
var getPoints = function getPoints(colorList, offset) {
    var startPoint = colorList[0];
    var endPoint = colorList[colorList.length - 1];
    var startIndex = 0;

    for (var i = 0; i < colorList.length; i++) {
        if (colorList[i].offset < offset) {
            startPoint = colorList[i];
            startIndex = i;
            continue;
        } else {
            endPoint = colorList[i];
            break;
        }
    }

    return {
        startPoint: startPoint,
        endPoint: endPoint,
        startIndex: startIndex
    };
};

var addColorAlpha = function addColorAlpha(color, opacity) {
    return Object.assign(color, {
        a: +opacity.toFixed(1)
    });
};

var getEndPoint = function getEndPoint(inx, datas, key) {
    var result = null;
    for (var i = inx; i < datas.length; i++) {
        if (key === 'color' && datas[i].color.r !== null) {
            result = datas[i];
        }

        if (key === 'alpha' && datas[i].color.a !== null) {
            result = datas[i];
        }
    }

    return result;
};

function getMediumColor(_ref) {
    var startColor = _ref.startColor,
        endColor = _ref.endColor,
        offset = _ref.offset,
        _ref$hasAlpha = _ref.hasAlpha,
        hasAlpha = _ref$hasAlpha === undefined ? true : _ref$hasAlpha;

    var newColor = {
        r: 255,
        g: 255,
        b: 255
    };

    for (var key in startColor) {
        if (!hasAlpha && key === 'a') {
            continue;
        }
        var range = endColor[key] - startColor[key];
        newColor[key] = startColor[key] + range * offset;
    }

    return newColor;
};

function getMediumAlpha(_ref2) {
    var startAlpha = _ref2.startAlpha,
        endAlpha = _ref2.endAlpha,
        offset = _ref2.offset;

    var range = endAlpha - startAlpha;
    return startAlpha + range * offset;
};
function getTargetGradientColor(option) {
    var colorList = option.colorList,
        startColor = option.startColor,
        endColor = option.endColor,
        offset = option.offset;

    if (colorList && colorList.length && offset) {
        var _getPoints = getPoints(colorList, offset),
            startPoint = _getPoints.startPoint,
            endPoint = _getPoints.endPoint,
            startIndex = _getPoints.startIndex;

        var newOffset = +((offset - startPoint.offset) / (endPoint.offset - startPoint.offset)).toFixed(2);

        var color = getMediumColor({
            startColor: startPoint.color,
            endColor: endPoint.color,
            offset: newOffset
        });

        return {
            color: color,
            index: startIndex + 1
        };
    }

    if (startColor && endColor) {
        return {
            color: getMediumColor({
                startColor: startColor,
                endColor: endColor,
                offset: offset
            })
        };
    }
};

function completeRGBA(datas) {
    datas.forEach(function (data, inx) {
        if (data.color.r === null && data.color.a) {
            var startPoint = datas[inx - 1];
            var endPoint = getEndPoint(inx + 1, datas, 'color');
            var currentColor = getMediumColor({
                startColor: startPoint.color,
                endColor: endPoint.color,
                offset: (data.offset - startPoint.offset) / (endPoint.offset - startPoint.offset),
                hasAlpha: false
            });

            data.color = addColorAlpha(currentColor, data.color.a);
        }

        if (data.color.a === null) {
            var _startPoint = datas[inx - 1];
            var _endPoint = getEndPoint(inx + 1, datas, 'alpha');
            var currentAlpha = getMediumAlpha({
                startAlpha: _startPoint.color.a,
                endAlpha: _endPoint.color.a,
                offset: (data.offset - _startPoint.offset) / (_endPoint.offset - _startPoint.offset)
            });

            data.color = addColorAlpha(data.color, currentAlpha);
        }
    });

    return datas;
}

function mergeColorAndAlpha(colorList, alphaList) {
    var newList = [];

    var pColor = 0;
    var pAlpha = 0;

    do {
        if (pColor >= colorList.length) {
            newList.push(alphaList[pAlpha]);
            pAlpha++;
            continue;
        }

        if (pAlpha >= alphaList.length) {
            newList.push(colorList[pColor]);
            pColor++;
            continue;
        }

        if (colorList[pColor].offset > alphaList[pAlpha].offset) {
            newList.push(alphaList[pAlpha]);
            pAlpha++;
            continue;
        }

        if (colorList[pColor].offset < alphaList[pAlpha].offset) {
            newList.push(colorList[pColor]);
            pColor++;
            continue;
        }

        if (colorList[pColor].offset === alphaList[pAlpha].offset) {
            var color = addColorAlpha(colorList[pColor].color, alphaList[pAlpha].color.a);
            newList.push({
                color: color,
                offset: colorList[pColor].offset
            });
            pColor++;
            pAlpha++;
        }
    } while (pColor < colorList.length || pAlpha < alphaList.length);

    return completeRGBA(newList);
};