'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _intLimit = require('./int-limit');

var _intLimit2 = _interopRequireDefault(_intLimit);

var _degreeToRadius = require('./degree-to-radius');

var _degreeToRadius2 = _interopRequireDefault(_degreeToRadius);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var rectTools = {
    getBBox: function getBBox(rect, zoom) {
        var rectRotate = (0, _intLimit2.default)(rect.rotate || 0, 360);
        var rectHeight = rect.height;
        var rectWidth = rect.width;
        var rotate = rectRotate > 90 && rectRotate < 180 || rectRotate > 270 && rectRotate < 360 ? 180 - rectRotate : rectRotate;

        var rad = (0, _degreeToRadius2.default)(rotate);
        var height = Math.abs(Math.sin(rad) * rectWidth + Math.cos(rad) * rectHeight);
        var width = Math.abs(Math.sin(rad) * rectHeight + Math.cos(rad) * rectWidth);

        var dotX = rect.left + rectWidth / 2;
        var dotY = rect.top + rectHeight / 2;
        var left = dotX - width / 2;
        var top = dotY - height / 2;

        zoom = zoom || 1;

        return {
            rotate: rotate,
            height: height * zoom,
            width: width * zoom,
            left: left * zoom,
            top: top * zoom
        };
    },

    getElementRect: function getElementRect(element, zoom) {
        if (!zoom) {
            zoom = 1;
        }

        var rect = {
            padding: [0, 0, 0, 0],
            height: zoom * element.height,
            width: zoom * element.width,
            left: zoom * element.left,
            top: zoom * element.top,
            clip: {
                bottom: 0,
                right: 0,
                left: 0,
                top: 0
            }
        };

        if (element.padding) {
            Object.keys(element.padding).forEach(function (key) {
                rect.padding[key] = zoom * element.padding[key];
            });
        }

        if (element.clip) {
            Object.keys(element.clip).forEach(function (key) {
                rect.clip[key] = zoom * element.clip[key];
            });
        }

        rect.rotate = element.rotate;

        return rect;
    },

    getBBoxByElement: function getBBoxByElement(element, zoom) {
        var rect = rectTools.getElementRect(element, zoom);

        return rectTools.getBBox(rect);
    },
    getBBoxByElements: function getBBoxByElements(elements, zoom) {
        var top = Infinity;
        var left = Infinity;
        var right = -Infinity;
        var bottom = -Infinity;

        elements.forEach(function (element) {
            var bbox = rectTools.getBBoxByElement(element, zoom);

            if (bbox.top < top) {
                top = bbox.top;
            }
            if (bbox.left < left) {
                left = bbox.left;
            }
            if (bbox.left + bbox.width > right) {
                right = bbox.left + bbox.width;
            }
            if (bbox.top + bbox.height > bottom) {
                bottom = bbox.top + bbox.height;
            }
        });

        return {
            rotate: 0,
            height: bottom - top,
            width: right - left,
            left: left,
            top: top
        };
    }
};

exports.default = rectTools;
module.exports = exports['default'];